<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:jython-extension="http://www.jython.org/"
  xmlns:redirect="org.apache.xalan.xslt.extensions.Redirect"
  extension-element-prefixes="redirect"
  xmlns:lxslt="http://xml.apache.org/xslt"
  exclude-result-prefixes="lxslt">
  
  <xsl:output method="html"/>

  <xsl:strip-space elements="*"/>

  <xsl:template name="js">
    <xsl:text>
        function suppress_errors ()
        {
          return true;
        }

        function does_element_exist (svg_name, element_name)
        {
          // First, redirect the error handler so that if the SVG plug-in has 
          // not yet loaded or is not present, it doesn't cause the browser to
          // issue a JavaScript error.
          var old_error = window.onerror;
          window.onerror = suppress_errors;

          // Now attempt to get the SVG object.
          var svgobj = document.embeds[svg_name].
              getSVGDocument().getElementById(element_name);
            
          // Reset the error handler to the browser's default handler.
          window.onerror = old_error;

          // Return appropriate value.
          if (svgobj == null)
            return false;
          else
            return true;
        }

        function mouse_over (target_id)
        {
          var svgdoc = document.pie.getSVGDocument();
          var svgobj;
          var svgstyle;

          var detail_name = 'details' + target_id;
          svgobj = svgdoc.getElementById(detail_name);
          if (svgobj != null)
          {
            svgstyle = svgobj.getStyle();
            svgstyle.setProperty ('visibility', 'visible');
          }
    </xsl:text>
    <xsl:for-each select="/sales/region">
      <xsl:text>svgobj = svgdoc.getElementById('legend</xsl:text>
      <xsl:value-of select="position()"/><xsl:text>');</xsl:text>
      <xsl:text>
          if (svgobj != null)
          {
            svgstyle = svgobj.getStyle();
            svgstyle.setProperty ('visibility', 'hidden');
          }
      </xsl:text>
    </xsl:for-each>
    <xsl:text>
          // Propagate the event to other handlers.
          return true;
        }

        function mouse_out ()
        {
          var svgdoc = document.pie.getSVGDocument();
          var svgobj;
          var svgstyle;

    </xsl:text>
    <xsl:for-each select="/sales/region">
      <xsl:text>svgobj = svgdoc.getElementById('legend</xsl:text>
      <xsl:value-of select="position()"/><xsl:text>');</xsl:text>
      <xsl:text>
          if (svgobj != null)
          {
            svgstyle = svgobj.getStyle();
            svgstyle.setProperty ('visibility', 'visible');
          }
      </xsl:text>
      <xsl:text>svgobj = svgdoc.getElementById('details</xsl:text>
      <xsl:value-of select="position()"/><xsl:text>');</xsl:text>
      <xsl:text>
          if (svgobj != null)
          {
            svgstyle = svgobj.getStyle();
            svgstyle.setProperty ('visibility', 'hidden');
          }
      </xsl:text>
    </xsl:for-each>
    <xsl:text>
          // Propagate the event to other handlers.
            return true;
        }
    </xsl:text>
  </xsl:template>

  <lxslt:component prefix="jython-extension" functions="cos sin toRadians">
    <lxslt:script lang="jpython">
import math

def cos(d):
  return math.cos(d)

def sin(d):
  return math.sin(d)

def toRadians(d):
  return d / 180 * math.pi
    </lxslt:script>
  </lxslt:component>

  <xsl:template match="/">
    <xsl:apply-templates select="sales"/>
  </xsl:template>

  <xsl:template match="sales">
    <html>
      <head>
        <title>
          <xsl:value-of select="caption/heading"/>
        </title>
        <script language="JavaScript1.2">
          <xsl:comment>
            <xsl:call-template name="js"/>
            <xsl:text>// </xsl:text></xsl:comment>
        </script>
      </head>
      <body>
        <center>
          <embed name="pie" width="650" height="500" src="saleschart.svg"/>
        </center>
      </body>
    </html>
    <redirect:write select="concat('sales', 'chart', '.svg')">
      <svg width="450" height="300">
        <text style="font-size:24; text-anchor:middle; 
          font-weight:bold" x="130" y="20">
          <xsl:value-of select="caption/heading"/>
        </text>
        <text style="font-size:14; text-anchor:middle" y="40" x="130">
          <xsl:value-of select="caption/subheading"/>
        </text>
        
        <xsl:variable name="totalSales" select="sum(//product)"/>
        
        <xsl:for-each select="region">
          <xsl:variable name="regionSales" select="sum(product)"/>
          <xsl:variable name="color">
            <xsl:choose>
              <xsl:when test="(position() mod 6) = 1">
                <xsl:text>red</xsl:text>
              </xsl:when>
              <xsl:when test="(position() mod 6) = 2">
                <xsl:text>orange</xsl:text>
              </xsl:when>
              <xsl:when test="(position() mod 6) = 3">
                <xsl:text>purple</xsl:text>
              </xsl:when>
              <xsl:when test="(position() mod 6) = 4">
                <xsl:text>blue</xsl:text>
              </xsl:when>
              <xsl:when test="(position() mod 6) = 5">
                <xsl:text>green</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>orange</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:variable name="explode" select="position()=1"/>
          
          <xsl:apply-templates select=".">
            <xsl:with-param name="color" select="$color"/>
            <xsl:with-param name="regionSales" select="$regionSales"/>
            <xsl:with-param name="totalSales" select="$totalSales"/>
            <xsl:with-param name="runningTotal" 
              select="sum(preceding-sibling::region/product)"/>
            <xsl:with-param name="explode" select="$explode"/>
            <xsl:with-param name="position" select="position()"/>
          </xsl:apply-templates>          
          
          <xsl:apply-templates select="." mode="legend">
            <xsl:with-param name="color" select="$color"/>
            <xsl:with-param name="regionSales" select="$regionSales"/>
            <xsl:with-param name="y-legend-offset" 
              select="90 + (position() * 20)"/>
            <xsl:with-param name="position" select="position()"/>
          </xsl:apply-templates>
          
          <xsl:apply-templates select="." mode="details">
            <xsl:with-param name="color" select="$color"/>
            <xsl:with-param name="position" select="position()"/>
            <xsl:with-param name="y-legend-offset" select="110"/>
          </xsl:apply-templates>
          
        </xsl:for-each>
      </svg>
    </redirect:write>
  </xsl:template>

  <xsl:template match="region">
    <xsl:param name="color" select="'red'"/>
    <xsl:param name="runningTotal" select="'0'"/>
    <xsl:param name="totalSales" select="'0'"/>
    <xsl:param name="regionSales" select="'0'"/>
    <xsl:param name="explode"/>
    <xsl:param name="position" select="'1'"/>
    
    <xsl:variable name="currentAngle" 
      select="jython-extension:toRadians(($regionSales div 
                                          $totalSales) * 360.0)"/>

    <path style="fill:{$color}; stroke:black; stroke-width:2; 
      fillrule:evenodd; stroke-linejoin:bevel;"
      onmouseout="mouse_out();" onclick="return false;">
      <xsl:attribute name="transform">
        <xsl:choose>
          <xsl:when test="$explode">
            <xsl:text>translate(</xsl:text>
            <xsl:value-of 
              select="(jython-extension:cos($currentAngle div 2) * 20) + 100"/>
            <xsl:text>,</xsl:text>
            <xsl:value-of 
              select="(jython-extension:sin($currentAngle div 2) * -20) + 160"/>
              <xsl:text>) </xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>translate(100,160) </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text> rotate(</xsl:text>
        <xsl:value-of select="-1 * (($runningTotal div $totalSales) * 360.0)"/>
        <xsl:text>)</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="onmouseover">
        <xsl:text>return mouse_over(</xsl:text>
        <xsl:value-of select="$position"/><xsl:text>);</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="d">
        <xsl:text>M 80 0 A 80 80 0 </xsl:text>
        <xsl:choose>
          <xsl:when test="$currentAngle &gt; 3.14">
            <xsl:text>1 </xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>0 </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>0 </xsl:text>
        <xsl:value-of select="jython-extension:cos($currentAngle) * 80"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="jython-extension:sin($currentAngle) * -80"/>
        <xsl:text> L 0 0 Z </xsl:text>
      </xsl:attribute>
    </path>
  </xsl:template>

  <xsl:template match="region" mode="legend">
    <xsl:param name="color" select="'red'"/>
    <xsl:param name="regionSales" select="'0'"/>
    <xsl:param name="y-legend-offset" select="'0'"/>
    <xsl:param name="position" select="'1'"/>

    <g>
      <xsl:attribute name="id">
        <xsl:text>legend</xsl:text><xsl:value-of select="$position"/>
      </xsl:attribute>

      <text>
        <xsl:attribute name="style">
          <xsl:text>font-size:12; text-anchor:start</xsl:text> 
        </xsl:attribute> 
        <xsl:attribute name="x">
          <xsl:text>240</xsl:text> 
        </xsl:attribute>
        <xsl:attribute name="y">
          <xsl:value-of select="$y-legend-offset"/>
        </xsl:attribute>
        <xsl:value-of select="name"/>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="$regionSales"/>
        <xsl:text>) </xsl:text>
      </text>
      
      <path>
        <xsl:attribute name="style">
          <xsl:text>stroke:black; stroke-width:2; fill:</xsl:text> 
          <xsl:value-of select="$color"/>
        </xsl:attribute>
        <xsl:attribute name="d">
          <xsl:text>M 220 </xsl:text> 
          <xsl:value-of select="$y-legend-offset - 10"/>
          <xsl:text> L 220 </xsl:text> 
          <xsl:value-of select="$y-legend-offset"/>
          <xsl:text> L 230 </xsl:text> 
          <xsl:value-of select="$y-legend-offset"/>
          <xsl:text> L 230 </xsl:text> 
          <xsl:value-of select="$y-legend-offset - 10"/>
          <xsl:text> Z</xsl:text> 
        </xsl:attribute>
      </path>
    </g>
  </xsl:template>

  <xsl:template match="region" mode="details">
    <xsl:param name="color" select="black"/>
    <xsl:param name="position" select="'0'"/>
    <xsl:param name="y-legend-offset"/>

    <g style="visibility:hidden">
      <xsl:attribute name="id">
        <xsl:text>details</xsl:text><xsl:value-of select="$position"/>
      </xsl:attribute>
      <text style="font-size:14; font-weight:bold; 
        text-anchor:start; fill: {$color}" x="220">
        <xsl:attribute name="y">
          <xsl:value-of select="$y-legend-offset"/>
        </xsl:attribute>
        <xsl:value-of select="name"/><xsl:text> Sales:</xsl:text>
      </text>
      <xsl:for-each select="product">
        <text style="font-size:12; text-anchor:start" x="220">
          <xsl:attribute name="y">
            <xsl:value-of select="$y-legend-offset + (position() * 20)"/>
          </xsl:attribute>
          <xsl:value-of select="@name"/>
          <xsl:text>: </xsl:text><xsl:value-of select="."/>
        </text>
      </xsl:for-each>
    </g>
  </xsl:template>

</xsl:stylesheet>
